/* 
    Copyright (C) 2000 Marcus Metzler (mocm@convergence.de)
    for convergence integrated media

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
 
#include <stdint.h>
#include <strstream.h>
#include <time.h>
#include <ost/audio.h>
#include <ost/video.h>

#include "rtuxzap.hh"
#include "interface.hh"
extern "C"{
#ifdef HAVE_LIBLIRC_CLIENT
#include <lirc/lirc_client.h>
#endif
#include <curses.h>
}



DVB dvb;
ChannelList chlist[NLIST];
int clist = 0;
int lastclist = 0;
int lastlc[NLIST];
int currlc[NLIST];
int it = 0;
int oit = 1;
int dev = 0;
int o=0;
int audio=0;
time_t mytime=0;
int mute = 0;
boolean enable_dvr = false;
boolean wide = false;

#ifdef HAVE_LIBLIRC_CLIENT
static struct lirc_config *config;
#endif
int volume = 200;

void set_volume(int vol)
{
	int ans;
	audioMixer_t am;
	int audio_fd;
        char path[256];
	ostrstream str(path,256);
	
	if (enable_dvr) return;
	str << "/dev/ost/audio" << dev ;
	str << ends;

	if (vol > 255) vol = 255;
	if (vol < 0  ) vol = 0;
	volume = vol;

	am.volume_left = vol;
	am.volume_right = vol;

	if ((audio_fd = open(path, O_RDWR)) < 0){
	    perror("mixer");
	    return;
	}

	if ( (ans = ioctl(audio_fd,AUDIO_SET_MIXER, &am) < 0)){
		perror("AUDIO SET MIXER: ");
		return;
	}

	close (audio_fd);

	if (o){
		dvb.osd.Clear();

		ostrstream fstr;
		fstr << "Volume " << volume << ends;
		dvb.osd.Text(0, 0, 20, 1, fstr.str());
		dvb.osd.Show();
		mytime = time(0)+2;
	}
}

void set_format(videoFormat_t format)
{
        int ans;
	int fd;

        char path[256];
	ostrstream str(path,256);
	
	if (enable_dvr) return;
	str << "/dev/ost/video" << dev ;
	str << ends;

	if ((fd = open(path, O_RDWR)) < 0){
	    perror("display");
	    return;
	}

        if ( (ans = ioctl(fd,VIDEO_SET_FORMAT, format) < 0)){
                perror("VIDEO SET FORMAT: ");
        }
	close(fd);
}

void SetMute(int m){
	int audio_fd;
        char path[256];
	ostrstream str(path,256);
	audioMixer_t am;

	if (enable_dvr) return;
	mute = m;
	str << "/dev/ost/audio" << dev ;
	str << ends;

	if ((audio_fd = open(path, O_RDWR)) < 0){
	    perror("mixer");
	    return;
	}
	if (m){
		am.volume_left = 0;
		am.volume_right = 0;
	} else {
		am.volume_left = volume;
		am.volume_right = volume;
	}
	if ( ioctl(audio_fd,AUDIO_SET_MIXER, &am) < 0){
		perror("AUDIO SET MIXER: ");
	}
		
	close (audio_fd);
}


void stop_curses(void)
{
    clear();
    refresh();
    endwin();
}

void curses_init(void)     /* for kbd input */
{
    initscr();
    refresh();
    cbreak();
    noecho();
    nonl();
    intrflush(stdscr, FALSE);
    keypad(stdscr, TRUE);
    nodelay(stdscr, TRUE);

    atexit(stop_curses);
}

int lirc_zap_init()
{
#ifdef HAVE_LIBLIRC_CLIENT
	int fd;
	
	if (-1 == (fd = lirc_init("rtuxzap",0))) {
		cerr << "no infrared remote support available" << endl;
		return -1;
	}
	
	if (0 != lirc_readconfig(NULL,&config,NULL)) {
		lirc_deinit();
		return -1;
	}
	fcntl(fd,F_SETFL,O_NONBLOCK);
	fcntl(fd,F_SETFD,FD_CLOEXEC);
	
	return fd;
#else
	cerr << "no infrared remote support available" << endl;
	return -1;
#endif
}


int get_dvbrc(char *path,DVB &dv, int len)
{
        ifstream dvbin;

        const char *home = getenv("HOME");
        const char *file = ".dvbrc";
	ostrstream str(path,len-1);

	str << home << "/" << file ;
	if (dev)
		str << "." << dev ;
	str << ends;
	  
	dvbin.open(path);

        if (dvbin) {
                dvbin >> dv;
                return 1;
        }
        return 0;
}

static void get_dvbfav(char *path, int nb, int len)
{
        const char *home = getenv("HOME");
        const char *file = ".dvbfav";

	ostrstream str(path,len);
	
	str << home << "/" << file << nb ;
	if (dev)
		str << "." << dev;
	str << ends; 
	lastlc[nb] = 0;
	currlc[nb] = 0;

}

static void load_ch(int nb)
{
	char filen[80];

	get_dvbfav(filen,nb,80);
	cout << _("Reading channel list from ") << filen << endl;
	chlist[nb].init(filen);
}

static int cnum (gChannel & ch)
{
	char buf[80];
        ostrstream str(buf, 80);

        uint16_t onid  =  uint16_t((ch.DVB_Num() & 
				    0xFFFF000000000000ULL) >> 48);
	uint16_t satid =  uint16_t((ch.DVB_Num() & 
				    0x0000FFFF00000000ULL) >> 32);
        uint16_t tpid  =  uint16_t((ch.DVB_Num() & 
				    0x00000000FFFF0000ULL) >> 16);
	ushort sid     =  ushort   (ch.DVB_Num() & 0x000000000000FFFFULL);


	int chnr;
	int found=0;
	for (chnr=0; chnr< dvb.num[CHAN]; chnr++) {
		if (dvb.chans[chnr].pnr==sid   && 
		    (dvb.chans[chnr].onid==onid || onid == 0xffff) && 
		    (dvb.chans[chnr].satid==satid || satid == 0xffff)  && 
                    (dvb.chans[chnr].tpid==tpid)) {
		  found = 1;
		  break;
                }
	}
	if (!found) chnr = -1;
	return chnr;
}

int set_audio(int audion)
{
	int cchan= cnum(chlist[clist](currlc[clist]));
	int anum = dvb.chans[cchan].apidnum;

	if (enable_dvr) return 0;
	if (anum < 2) {
		if (o){
			dvb.osd.Clear();
			
			ostrstream fstr;
			fstr << "Only one Audio Channel" << ends;
			dvb.osd.Text(0, 0, 20, 1, fstr.str());
			dvb.osd.Show();
			mytime = time(0)+2;
		}
		return 0;
	}

	if (audion >= anum) audion = 0;
	audio = audion;
	if (o){
		dvb.osd.Clear();

		ostrstream fstr;
		fstr << "Audio " << audion << ends;
		dvb.osd.Text(0, 0, 20, 1, fstr.str());
		dvb.osd.Show();
		mytime = time(0)+2;
	}
	dvb.set_apid(dvb.chans[cchan].apids[audion]);  
	return 0;
}


void set_channel()
{
	int num = cnum(chlist[clist](currlc[clist]));
	
	if (num < 0) return;

	char buf[80];
	ostrstream str(buf, 80);
	str << _("Setting ") 
	    << dvb.chans[num].name << " ("<< dvb.chans[num].pnr 
	    << ")" << ends;

	SetMute(1);
	if (o && !enable_dvr ){
			ostrstream fstr;
			fstr << "F" << clist << "  Prog: " << currlc[clist] 
			     << ends;
			dvb.osd.Clear();
			dvb.osd.Text(0, 0, 20,
				     1,
				     fstr.str()
				);
			dvb.osd.Show();
			usleep(500);
	}

	audio = 0;
	dvb.SetChannel(num);
	mytime = time(0)+2;
	SetMute(0);
}


#define FILELEN 256
int main (int argc, char **argv)
{
	char filen[FILELEN];
	int c,channel = 0;
	int lfd= -1;
	struct pollfd pfd[2];
	boolean mrand = false;
	time_t rtime;

#ifdef ENABLE_NLS
	setlocale(LC_ALL,"");
	bindtextdomain("rtuxzap", LOCALE_DIR);
	textdomain("rtuxzap");
#endif
	lfd = lirc_zap_init();

	pfd[0].fd = STDIN_FILENO;
	pfd[0].events = POLLIN;
	pfd[1].fd = lfd;
	pfd[1].events = POLLIN;


	set_format(VIDEO_FORMAT_4_3);

	for (;;) {
		if (-1 == (c = getopt(argc, argv, "red:c:ow")))
			break;
		switch (c) {
		case 'c':
			channel = atoi(optarg);
			break;
		case 'r':
			mrand = true;
			break;
		case 'd':
			dev = atoi(optarg);
			break;

		case 'o':
			o = 1;
			break;

		case 'e':
			enable_dvr = true;
			break;

		case 'w':
			set_format(VIDEO_FORMAT_16_9);
			wide = true;
			break;

		case 'h':
		default:
			cerr << _("usage: ") << argv[0] << _("  [ options ] ") 
			     <<	endl << endl << _("options:") << endl
			     << endl << _("    -c <n>    set channel") << endl
			     << endl << _("    -d <n>    set devices (0 =/dev/ost/xxx0)") << endl
			     << endl << _("    -o        use OSD") << endl
			     << endl <<   "    -e        enable dvr" << endl
			     << endl <<   "    -w        set 16:9" << endl
			     << endl <<   "    -r        random zap" << endl;
			exit(1);
		}
	}
	
	dvb.init("","",dev);
	if (o) dvb.use_osd(dev);
	if (enable_dvr) dvb.enable_DVR();
	else dvb.disable_DVR();

	if (get_dvbrc(filen,dvb,FILELEN)) {
		cout << _("Found DVB input file ") << filen << endl;
	} else {
		cout << _("Could not find DVB input file ") 
		     << filen << endl;
	}

	for (int i = 0; i < NLIST; i++){
		load_ch(i);
	}

	curses_init();
	set_volume(volume);
	printf("Welcome to RTuxZap\r\n\n");
	refresh();


	set_channel();
	int progn = -1;
	if (mrand){
		rtime = time(0);
		srand(time(0));
	}
	for (;;){
		int sw;
		int cc = 0;
		char *code;
		char *c;
		time_t key_time;

		refresh();
		if (o && mytime &&time(0) > mytime){
			dvb.osd.Hide();
			mytime = 0;
		}
		if (mrand && rtime && rtime < time(0)){
			clist = rand()%NLIST; 
			currlc[clist] = rand()%(chlist[clist].NumChan());
			set_channel();
			rtime = time(0)+2;
		}
		
		if (progn >= 0 && time(0) > key_time){
			if (progn >= 0 && progn < chlist[clist].NumChan()){
				lastlc[clist] = currlc[clist];
				currlc[clist] = progn;
				set_channel();
			}
			lastclist = clist;
			progn = -1;
		}

		if (poll(pfd,2,1000)){
			if (pfd[0].revents & POLLIN){
				cc = getch();
			}
#ifdef HAVE_LIBLIRC_CLIENT
			if (pfd[1].revents & POLLIN){
				if ( lirc_nextcode(&code) == 0 &&
				     lirc_code2char(config,code,&c) ==0 &&
				     c!=NULL){
					//printf("%s\r\n",code);
					refresh();
					cc = int(c[0]); 
					
					free(code);
				}
			}
#endif
			if (cc){
				//printf("Key: %d\r\n",cc);
				switch(cc){
				case 'e':
					enable_dvr = !enable_dvr;

					if (enable_dvr)
						dvb.enable_DVR();
					else 
						dvb.disable_DVR();

					set_channel();
					
					if (o){
						ostrstream fstr;
						
						fstr << "dvr";
						if (enable_dvr)
							fstr << " enabled";
						else 
							fstr << " disabled";
						fstr << ends;
						dvb.osd.Clear();
						dvb.osd.Text(0, 0, 20,
							     1,
							     fstr.str()
							);
						dvb.osd.Show();
						mytime = time(0)+3;
					}  
					
					break;

				case 'w':
					wide = !wide;

					if (wide)
						set_format(VIDEO_FORMAT_16_9);
					else 
						set_format(VIDEO_FORMAT_4_3);
					
					if (o){
						ostrstream fstr;
						
						fstr << "wide screen";
						if (wide)
							fstr << " enabled";
						else 
							fstr << " disabled";
						fstr << ends;
						dvb.osd.Clear();
						dvb.osd.Text(0, 0, 20,
							     1,
							     fstr.str()
							);
						dvb.osd.Show();
						mytime = time(0)+3;
					}  
					
					break;

				case 'z':
					mrand = !mrand;

					if (mrand){
						rtime = time(0);
						srand(time(0));
					} else 	rtime = 0;
					
					if (o){
						ostrstream fstr;
						
						fstr << "random";
						if (mrand)
							fstr << " enabled";
						else 
							fstr << " disabled";
						fstr << ends;
						dvb.osd.Clear();
						dvb.osd.Text(0, 0, 20,
							     1,
							     fstr.str()
							);
						dvb.osd.Show();
						mytime = time(0)+3;
					}  
					
					break;


				case 'm':
					if (enable_dvr) break;
					SetMute(!mute);
					break;
				case 'q':
				case 'Q':
					if (enable_dvr) break;
					exit(0);
					break;
					
				case '+':
					if (enable_dvr) break;
					SetMute(0);
					lastclist = clist;
					set_volume(volume+1);
					break;

				case '-':
					if (enable_dvr) break;
					SetMute(0);
					lastclist = clist;
					set_volume(volume-1);
					break;

				case 'd':
				case 258:
					if (enable_dvr) break;
					lastlc[clist] = currlc[clist];
					currlc[clist]--;
					if (currlc[clist] < 0) 
						currlc[clist] =  
							chlist[clist].NumChan()
							-1;
					set_channel();
					break;
				case 'u':
				case 259:
					if (enable_dvr) break;
					lastlc[clist] = currlc[clist];
					currlc[clist]++;
					if (currlc[clist] >=
					    chlist[clist].NumChan())
						currlc[clist] = 0; 
					set_channel();
					break;

				case 261:
				case 'f':
					if (enable_dvr) break;
					lastclist = clist;
					clist++;
					if (clist > NLIST-1) clist = 0;
					set_channel();
					break;
					
				case 260:
				case 'g':
					lastclist = clist;
					clist = clist;
					clist--;
					if (clist < 0) clist = NLIST-1;
					set_channel();
					break;
					
				case 'a':
					if (enable_dvr) break;
					set_audio(audio+1);
					break;

				case 'r':
					if (enable_dvr) break;
					if (clist != lastclist){
						sw = clist;
						clist = lastclist;
						lastclist = sw;
					} else {
						sw = lastlc[clist];
						lastlc[clist] = currlc[clist];
						currlc[clist] = sw;
					}
					set_channel();
					break;
				
				case 't':
					if (enable_dvr) break;
					if (o){
						ostrstream fstr;
						time_t t;
						
						t = time(0);
						fstr << ctime(&t)
						     << ends;
						dvb.osd.Clear();
						dvb.osd.Text(0, 0, 20,
							     1,
							     fstr.str()
							);
						dvb.osd.Show();
						mytime = time(0)+2;
					}
					break;
					
				case '0'...'9':
					if (enable_dvr) break;
					if (progn >= 0)
						progn = 10*progn+(cc-48);
					else
						progn = cc-48;
					if (o){
						ostrstream fstr;
						time_t t;
						
						t = time(0);
						fstr << "Prog: "
						     << progn << ends;
						dvb.osd.Clear();
						dvb.osd.Text(0, 0, 20,
							     1,
							     fstr.str()
							);
						dvb.osd.Show();
						mytime = time(0)+1;
					}
					key_time = time(0)+1;
					break;
					
				}
			}
		}
	}
#ifdef HAVE_LIBLIRC_CLIENT
	lirc_freeconfig(config);
	lirc_deinit();
#endif
	exit (0);
}

