#ifndef _DVB_H_
#define _DVB_H_

#define DVB_FIRM_PATH "/lib/DVB/"

#include <linux/videodev.h>
#include <linux/i2c.h>
#include <linux/socket.h> 
#include <linux/netdevice.h> 

#ifdef CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
#endif

/* DEBI transfer mode defs */

#define DEBINOSWAP 0x000e0000
#define DEBISWAB   0x001e0000
#define DEBISWAP   0x002e0000

#define ARM_WAIT_FREE  (5*HZ)
#define ARM_WAIT_SHAKE (HZ/5)
#define ARM_WAIT_OSD (HZ)

#ifdef __KERNEL__

#if LINUX_VERSION_CODE < 0x020300
#define net_device device
#define DECLARE_MUTEX(foo)         struct semaphore foo = MUTEX
#define DECLARE_MUTEX_LOCKED(foo)  struct semaphore foo = MUTEX_LOCKED
#define WAIT_QUEUE                 struct wait_queue*
#define init_waitqueue_head(wq)    *(wq) = NULL;
#define DECLARE_WAITQUEUE(wait, current) struct wait_queue wait = { current, NULL }
#define set_current_state(state_value)			\
	do { current->state = state_value; } while (0)
#define list_for_each(pos, head) \
	for (pos = (head)->next; pos != (head); pos = pos->next)

#else

#define WAIT_QUEUE                 wait_queue_head_t

#endif

#include "dvbdev.h"
#ifdef __DVB_PACK__
#include <ost/video.h>
#include <ost/audio.h>
#include <ost/demux.h>
#include <ost/dmx.h>
#include <ost/sec.h>
#include <ost/frontend.h>
#include <ost/ca.h>
#include <ost/osd.h>
#include <ost/net.h>
#else
#include <linux/ost/video.h>
#include <linux/ost/audio.h>
#include <linux/ost/demux.h>
#include <linux/ost/dmx.h>
#include <linux/ost/sec.h>
#include <linux/ost/frontend.h>
#include <linux/ost/ca.h>
#include <linux/ost/osd.h>
#include <linux/ost/net.h>
#endif

#include "dvb_demux.h"
#include "dmxdev.h"
#include "dvb_demodulator.h"
#include "dvb_filter.h"
#include "dvb_net.h"


#define VIDIOCSPANSCAN          _IOW('v',  BASE_VIDIOCPRIVATE+8, int)
#define VID_NONE_PREF           0x00    /* No aspect ration processing preferred */
#define VID_PAN_SCAN_PREF       0x01    /* Pan and Scan Display preferred */
#define VID_VERT_COMP_PREF      0x02    /* Vertical compression display preferred */
#define VID_VC_AND_PS_PREF      0x03    /* PanScan and vertical Compression if allowed */
#define VID_CENTRE_CUT_PREF     0x05    /* PanScan with zero vector */

#define DVB_SET_FRONTEND        _IOW('v',  BASE_VIDIOCPRIVATE+0x10, struct frontend)
#define DVB_GET_FRONTEND        _IOR('v',  BASE_VIDIOCPRIVATE+0x11, struct frontend)
#define DVB_INIT_FRONTEND       _IOR('v',  BASE_VIDIOCPRIVATE+0x12, void)
#define DVB_RESET               _IOR('v',  BASE_VIDIOCPRIVATE+0x13, void)
#define DVB_WRITEREG            _IOR('v',  BASE_VIDIOCPRIVATE+0x14, u8 *)
#define DVB_READREG             _IOR('v',  BASE_VIDIOCPRIVATE+0x15, u8 *)


#define DATA_NONE                0x00 
#define DATA_FSECTION            0x01
#define DATA_IPMPE               0x02
#define DATA_MPEG_RECORD         0x03
#define DATA_DEBUG_MESSAGE       0x04
#define DATA_COMMON_INTERFACE    0x05
#define DATA_MPEG_PLAY           0x06
#define DATA_BMP_LOAD            0x07
#define DATA_IRCOMMAND           0x08
#define DATA_PIPING              0x09
#define DATA_STREAMING           0x0a

#define DATA_PES_RECORD          0x10
#define DATA_PES_PLAY            0x11
#define DATA_TS_RECORD           0x12
#define DATA_TS_PLAY             0x13


#define CI_CMD_ERROR             0x00
#define CI_CMD_ACK               0x01
#define CI_CMD_SYSTEM_READY      0x02
#define CI_CMD_KEYPRESS          0x03
#define CI_CMD_ON_TUNED          0x04
#define CI_CMD_ON_SWITCH_PROGRAM 0x05
#define CI_CMD_SECTION_ARRIVED   0x06
#define CI_CMD_SECTION_TIMEOUT   0x07
#define	CI_CMD_TIME              0x08
#define	CI_CMD_ENTER_MENU        0x09
#define	CI_CMD_FAST_PSI          0x0a
#define	CI_CMD_GET_SLOT_INFO     0x0b

#define	CI_MSG_NONE              0x00
#define	CI_MSG_CI_INFO           0x01
#define	CI_MSG_MENU              0x02
#define	CI_MSG_LIST              0x03
#define	CI_MSG_TEXT              0x04
#define	CI_MSG_REQUEST_INPUT     0x05
#define	CI_MSG_INPUT_COMPLETE    0x06
#define	CI_MSG_LIST_MORE         0x07
#define	CI_MSG_MENU_MORE         0x08
#define	CI_MSG_CLOSE_MMI_IMM     0x09
#define	CI_MSG_SECTION_REQUEST   0x0a
#define	CI_MSG_CLOSE_FILTER      0x0b
#define	CI_PSI_COMPLETE          0x0c
#define	CI_MODULE_READY          0x0d
#define	CI_SWITCH_PRG_REPLY      0x0e
#define	CI_MSG_TEXT_MORE         0x0f

#define	CI_MSG_CA_PMT            0xe0
#define	CI_MSG_ERROR             0xf0


typedef struct dvb_buf {
        int length;
        u8 *buf;
	spinlock_t lock;
	int head;
        int tail;
	WAIT_QUEUE queue;
	struct semaphore sema;
} dvb_buf_t;


#define PROG_STREAM_MAP  0xBC
#define PRIVATE_STREAM1  0xBD
#define PADDING_STREAM   0xBE
#define PRIVATE_STREAM2  0xBF
#define AUDIO_STREAM_S   0xC0
#define AUDIO_STREAM_E   0xDF
#define VIDEO_STREAM_S   0xE0
#define VIDEO_STREAM_E   0xEF
#define ECM_STREAM       0xF0
#define EMM_STREAM       0xF1
#define DSM_CC_STREAM    0xF2
#define ISO13522_STREAM  0xF3
#define PROG_STREAM_DIR  0xFF

#define PTS_DTS_FLAGS    0xC0

//pts_dts flags 
#define PTS_ONLY         0x80
#define PTS_DTS          0xC0
#define TS_SIZE        188
#define TRANS_ERROR    0x80
#define PAY_START      0x40
#define TRANS_PRIO     0x20
#define PID_MASK_HI    0x1F
//flags
#define TRANS_SCRMBL1  0x80
#define TRANS_SCRMBL2  0x40
#define ADAPT_FIELD    0x20
#define PAYLOAD        0x10
#define COUNT_MASK     0x0F

// adaptation flags
#define DISCON_IND     0x80
#define RAND_ACC_IND   0x40
#define ES_PRI_IND     0x20
#define PCR_FLAG       0x10
#define OPCR_FLAG      0x08
#define SPLICE_FLAG    0x04
#define TRANS_PRIV     0x02
#define ADAP_EXT_FLAG  0x01

// adaptation extension flags
#define LTW_FLAG       0x80
#define PIECE_RATE     0x40
#define SEAM_SPLICE    0x20

#define MAX_PLENGTH    0xFFFF
#define MAX_VID_PES    0x1FFF
	
typedef struct section_s {
	int                    id;
	int                    length;
	int                    found;
	u8                     payload[4096+3];
} section_t;


#define MY_STATE_PES_START     1
#define MY_STATE_PES_STARTED   2
#define MY_STATE_FULL          4

#define MASKL   DMX_MAX_FILTER_SIZE
#define MAXFILT 32

#define QPSK_MAX_EVENT 8
#define QAM_MAX_EVENT 8

typedef struct qpsk_s {
        struct qpskEvent        events[QPSK_MAX_EVENT];
        int                     eventw;
        int                     eventr;
        int                     overflow;
        WAIT_QUEUE              eventq;
	spinlock_t              eventlock;
} qpsk_t;

typedef struct qam_s {
        struct qamEvent         events[QAM_MAX_EVENT];
        int                     eventw;
        int                     eventr;
        int                     overflow;
        WAIT_QUEUE              eventq;
	spinlock_t              eventlock;
} qam_t;

struct dvb_filter {
        int                    state;
        int                    flags;
        int                    type;
        u8                     ts_state;

        u16                    pid;
        u8                     value[MASKL];
        u8                     mask[MASKL];
};


enum {AV_PES_STREAM, PS_STREAM, TS_STREAM, PES_STREAM};

typedef struct ps_packet_s{
	u8                     scr[6];
	u8                     mux_rate[3];
	u8                     stuff_length;
	u8                     data[20];
	u8                     sheader_llength[2];
	int                    sheader_length;
	u8                     rate_bound[3];
	u8                     audio_bound;
	u8                     video_bound;
	u8                     reserved;
	int                    npes;
	int                    mpeg;
} ps_packet_t;

typedef struct a2p_s{
	int                    type;
	int                    found;
	int                    length;
	int                    headr;
	u8                     cid;
	u8                     flags;
	u8                     abuf[MAX_PLENGTH];
	int                    alength;
	u8                     vbuf[MAX_PLENGTH];
	int                    vlength;
	int                    plength;
	u8                     last_av_pts[4];
	u8                     av_pts[4];
	u8                     scr[4];
	u16                    count0;
	u16                    count1;
	u16                    pidv;
	u16                    pida;
	u16                    countv;
	u16                    counta;
        void                   *dataA;
        void                   *dataV;
        void                   (*write_cb)(u8 const *buf, long int count,
                                     void *data);
} a2p_t;


typedef struct p2t_s {
        u8               pes[TS_SIZE];
        u8               counter;
        long int         pos;
        int              frags;
        dvb_demux_feed_t *feed;
} p2t_t;


#define MAX_NUM_DVB 4
#define DVB_DEVS_MAX 16

typedef struct dvb_devs_s {
        int num;  
        int tab[DVB_DEVS_MAX];
        int max_users[DVB_DEVS_MAX];
        int max_writers[DVB_DEVS_MAX];
} dvb_devs_t;

struct frontend {
	int type;              /* type of frontend (tv tuner, dvb tuner/decoder, etc. */
#define FRONT_TV   0
#define FRONT_DVBS 1
#define FRONT_DVBC 2
#define FRONT_DVBT 3

	/* Sat line control */
        int power;             /* LNB power 0=off/pass through, 1=on */
	int volt;              /* 14/18V (V=0/H=1) */
	int ttk;               /* 22KHz */
	int diseqc;            /* Diseqc input select */
        
	/* signal decoding, transponder info */

	__u32 freq;            /* offset frequency (from local oscillator) in Hz */
        int AFC;
	__u32 srate;           /* Symbol rate in Hz */
	int qam;               /* QAM mode for cable decoder, sat is always QPSK */
        int inv;               /* Inversion */
	int fec;               /* Forward Error Correction */

        /* channel info */
	__u16 video_pid;        
	__u16 audio_pid;
        __u16 tt_pid;          /* Teletext PID */
        __u16 pcr_pid;          /* Teletext PID */
        __u16 pnr;             /* Program number = Service ID */

	/* status information */

	int fsync;             /* frequency sync (from tuner) */
        __u32 curfreq;         /* frequency which is actually used, e.g. after AFC */
        
	int sync;              /* sync from decoder */
#define DVB_SYNC_SIGNAL        1
#define DVB_SYNC_CARRIER       2
#define DVB_SYNC_VITERBI       4
#define DVB_SYNC_FSYNC         8
#define DVB_SYNC_FRONT        16
	__s32 afc;               /* frequency offset in Hz */
	__u16 agc;             /* gain */
	__u16 nest;            /* noise estimation */
        __u32 vber;            /* viterbi bit error rate */
	__u16 mode;		/* mode byte, including vber scale factor */

	int flags;
#define FRONT_TP_CHANGED  1 
#define FRONT_FREQ_CHANGED 2
#define FRONT_RATE_CHANGED 4
};

/* place to store all the necessary device information */
struct dvb_struct {
	struct video_device	video;
	int                     num;

	/* devices */

	struct saa7146          *saa;
	struct i2c_adapter      *i2cbus;
	struct i2c_client       *tuner;
	struct i2c_client       *demod;
	struct i2c_client       *sound;
	
	int dvbtype;
#define DVB_NONE 0
#define DVB_S    1
#define DVB_C    2
#define DVB_T    3

        int card_type;           /* basic card type */
#define DVB_CARD_TT_SIEMENS   0
#define DVB_CARD_TT_BUDGET    1
        int board_rev;           /* board revision */
        int demod_type;          /* demodulator type */
#define DVB_DEMOD_NONE    0
#define DVB_DEMOD_VES1893 1
#define DVB_DEMOD_VES1820 2
#define DVB_DEMOD_STV0299 3
#define DVB_DEMOD_TDA8083 4
        int tuner_type;          /* as define in tuner.h */


	/* buffers */
        void                   *iobuf;   // memory for all buffers
        struct dvb_buf          avout;
#define AVOUTLEN (256*1024)
        struct dvb_buf          aout;
#define AOUTLEN (64*1024)


	/* bitmap buffers and states */

        void                    *bmpbuf;
        int                     bmpp;
        int                     bmplen;
        int                     bmp_win;
        u16                     bmp_x, bmp_y;
        int                     bmp_trans;
        int                     bmp_state;
#define BMP_NONE     0
#define BMP_LOADING  1
#define BMP_LOADINGS 2
#define BMP_LOADED   3
	WAIT_QUEUE              bmpq;


	/* DEBI and polled command interface */

        spinlock_t              debilock;
	struct semaphore        dcomlock;
	WAIT_QUEUE              debiq;
	int                     debitype;
	int                     debilen;
	int                     debibuf;


        /* frontend monitor thread */

	struct task_struct      *mon_thread;
        WAIT_QUEUE              mon_wait;
        struct semaphore        mon_sem;
        int                     mon_tuning;
        int                     mon_rmmod;
        int                     mon_fstate;
        unsigned long           mon_delay;
	struct timer_list       mon_timer;
        int                     mon_lock;

        u32                     new_freq;
        u32                     new_srate;
        int                     new_fec;
        int                     new_qam;

	struct                  frontend front;
        

        /* Recording and playback flags */
        
	int                     rec_mode;
	int                     playing;
#define RP_NONE  0
#define RP_VIDEO 1
#define RP_AUDIO 2
#define RP_AV    3

        int                     osdwin;      /* currently active window */
        u16                     osdbpp[8];

        int                     ci_slot[2];
#define CI_MODULE_PRESENT 1
#define CI_MODULE_OK      2
        int                     ci_stat;

        int                     vidmode;
        dmxdev_t                dmxdev;
        dvb_demux_t             demux;
        char                    demux_id[16];

        dmx_frontend_t          hw_frontend;
        dmx_frontend_t          mem_frontend;

        qpsk_t                  qpsk;
        qam_t                   qam;
        int                     secbusy;

        boolean                 video_blank;
        struct videoStatus      videostate;
        int                     display_ar;
        int                     trickmode;
#define TRICK_NONE   0
#define TRICK_FAST   1
#define TRICK_SLOW   2
#define TRICK_FREEZE 3
        struct audioStatus      audiostate;
        u32                     powerstate;

        dvb_demux_filter_t      *handle2filter[32];
        p2t_t                   p2t_filter[MAXFILT];
        pes2ts_t                p2t[2];
        struct ipack_s          ipack[2]; 
        u8                      *kbuf[2];
        
        int sinfo;
        int shsize;
        int swsize;

        int tsf;
        int arm_errors;
        int feeding;
        int registered;

        
        /* DVB device */

        struct dvb_device dvb_dev;
        dvb_devs_t *dvb_devs;
        int users[DVB_DEVS_MAX];
        int writers[DVB_DEVS_MAX];
        dvb_net_t dvb_net;

        /* AV711X */

        u32 arm_fw;
        u32 arm_rtsl;
        u32 arm_vid;
        u32 arm_app;
        u32 avtype;
};

#endif

#endif
/*
 * Local variables:
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -8
 * c-argdecl-indent: 8
 * c-label-offset: -8
 * c-continued-statement-offset: 8
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
